from __future__ import print_function # In python 2.7
import sys
from flask import Flask
from flask import request
from api_request_manager import *
from api_user_weights import *
from api_response import ApiResponse
from data_holder import DataHolder
from cartodb import CartoDBAPIKey, CartoDBException
import time
import os

app = Flask(__name__)
app.debug = True

app.config.update(dict(
    DEBUG=True,
    PRODUCTION=True,
    cacheFileFolder = "/tmp/ram/",
    API_KEY = 'a50f67b019a990a1f29893ca6beab24028c2aea0',
))

app.config.from_envvar('FLASKR_SETTINGS', silent=True)



@app.teardown_appcontext
def closeAllDb(error):
    printConsole("Closing")

def printConsole(string):
    print(string, file=sys.stderr)

@app.route('/',methods=["POST","GET"])
def default():
    return "root"

@app.route('/clear_cache',methods=["POST","GET"])
def clear_cache():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    clear_db()
    clear_files()
    return apiResponse.getJson()

@app.route('/clear_db_cache',methods=["POST","GET"])
def clear_db():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    deleteFromCarto(table="results_zivotni_prostredi")
    deleteFromCarto(table="results_socialni")
    deleteFromCarto(table="results_nemovitosti")
    deleteFromCarto(table="results_sluzby")
    deleteFromCarto(table="results_doprava")
    deleteFromCarto(table="results_all")
    return apiResponse.getJson()

@app.route('/clear_file_cache',methods=["POST","GET"])
def clear_files():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    os.system("rm -fr "+app.config['cacheFileFolder']+"*"+".bin")
    return apiResponse.getJson()


@app.route('/info',methods=["POST","GET"])
def info():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    requestManager = RequestManager(request)
    kod = int(requestManager.getFromGet('kod'))
    table = "results_all"
    page = "generate_map"
    select_keys = "blizkost_kontaminace,hluk_2012,plocha_zelene_osm,frekventovana_silnice_score,znecisteni_ovzdusi_prekroceni,zaplavove_uzemi_aktivni_score,cena_nemovitosti,pomer_obytnych_budov,okolni_pocet_podlazi,plocha_brownfiled,v_obytne_zone,v_residencni_ulici,internet, hustota_zalidneni, podil_romske_narodnosti, podil_postproduktivni_obyvatel, facebook_navstevnost, kriminalita, pocitova_mapa, recyklokontejner_vzdalenost, posta_vzdalenost, potraviny_vzdalenost, hospodybary_vzdalenost, obchody_plocha, restaurace_vzdalenost, mhd_vzdalenost, mhd_ceska, mhd_mendlak, mhd_skacelova"
    cache_file_path = app.config['cacheFileFolder'] + page + ".bin"
    data = DataHolder(select_keys=select_keys, special_keys="kod,the_geom", api_key=app.config['API_KEY'])

    if not data.loadData(cacheFile=cache_file_path):
        apiResponse.setStatus(status=ApiResponse.error)
        apiResponse.setReason(reason="Data not in cache, please draw map first")
        return apiResponse.getJson()


    zivotni_prostredi_keys = ['hluk_2012','frekventovana_silnice_score','znecisteni_ovzdusi_prekroceni','plocha_zelene_osm','recyklokontejner_vzdalenost']
    socialni_statut_keys = ['cena_nemovitosti', 'pomer_obytnych_budov', 'okolni_pocet_podlazi', 'plocha_brownfiled', 'v_obytne_zone', 'v_residencni_ulici','pocitova_mapa']
    bezpeci_keys = ['kriminalita']
    relaxace_keys = ['facebook_navstevnost', 'hospodybary_vzdalenost', 'restaurace_vzdalenost']
    sluzby_keys = ['posta_vzdalenost', 'potraviny_vzdalenost', 'hospodybary_vzdalenost', 'obchody_plocha']
    doprava_keys = ['mhd_vzdalenost', 'mhd_ceska', 'mhd_mendlak', 'mhd_skacelova']

    data.data["index_zivotni_prostredi"] = data.getNormalizedSumForKeys(keys=zivotni_prostredi_keys)
    data.data["index_socialni_statut"] = data.getNormalizedSumForKeys(keys=socialni_statut_keys)
    data.data["index_bezpeci"] = data.getNormalizedSumForKeys(keys=bezpeci_keys)
    data.data["index_relaxace"] = data.getNormalizedSumForKeys(keys=relaxace_keys)
    data.data["index_nakupy"] = data.getNormalizedSumForKeys(keys=sluzby_keys)
    data.data["index_doprava"] = data.getNormalizedSumForKeys(keys=doprava_keys)

    data.calculateQuantile(user_except_keys=[])

    out_data = data.getDataForKod(kod)
    bins_data = data.data_bins

    apiResponse.setBins(bins=bins_data)
    apiResponse.setDetails(details=out_data)
    return apiResponse.getJson()

@app.route('/generate_map',methods=["POST","GET"])
def generate_map():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    try:
        requestManager = RequestManager(request)
        table = "results_all"
        page = "generate_map"
        userWeights = UserWeights(requestManager)
        userWeights.setUserWeightsAll()
    except Exception as e:
        printConsole(e)
        apiResponse.setStatus(status=ApiResponse.error)
        apiResponse.setReason(reason="Not valid GET parameters or input data")
        return apiResponse.getJson()

    #try:self.data
    select_keys = "blizkost_kontaminace,hluk_2012,plocha_zelene_osm,frekventovana_silnice_score,znecisteni_ovzdusi_prekroceni,zaplavove_uzemi_aktivni_score,cena_nemovitosti,pomer_obytnych_budov,okolni_pocet_podlazi,plocha_brownfiled,v_obytne_zone,v_residencni_ulici,internet, hustota_zalidneni, podil_romske_narodnosti, podil_postproduktivni_obyvatel, facebook_navstevnost, kriminalita, pocitova_mapa, recyklokontejner_vzdalenost, posta_vzdalenost, potraviny_vzdalenost, hospodybary_vzdalenost, obchody_plocha, restaurace_vzdalenost, mhd_vzdalenost, mhd_ceska, mhd_mendlak, mhd_skacelova"
    except_remove_outliners = ["blizkost_kontaminace","znecisteni_ovzdusi_prekroceni","zaplavove_uzemi_aktivni_score","v_obytne_zone","internet","v_residencni_ulici","pocitova_mapa"]
    except_reverse_scale =  ["plocha_zelene_osm","pomer_obytnych_budov","v_obytne_zone","internet","v_residencni_ulici","facebook_navstevnost","pocitova_mapa","hustota_zalidneni","obchody_plocha"]
    cache_file_path =  app.config['cacheFileFolder'] + str(page) + ".bin"
    data = DataHolder(select_keys=select_keys, api_key=app.config['API_KEY'])
    data.setUserWeights(userWeights=userWeights.getUserWeightsAll())

    # check wheather is already in db
    if checkIfHashExists(hash=data.userWeightsToHash(),table=table):
        apiResponse.setHash(hash=data.userWeightsToHash())
        apiResponse.setFromCahce(cache=ApiResponse.yes)
        return apiResponse.getJson()
    else:
        hash = data.userWeightsToHash()

        if not data.loadData(cacheFile=cache_file_path):
            data.downloadFromCarto()
            data.removeOutliners(user_except_keys=except_remove_outliners)
            data.standardizeData()
            data.normalizeData()
            data.reverseDataScale(user_except_keys=except_reverse_scale)
            data.saveData(cacheFile=cache_file_path)

        # WeightsSum
        data.runWeightsSum()
        # WPCA
        data.setWPCAWeights()
        data.runWPCA()
        # normalize output

        data.normalizeOutput(normalize=True, removeOutliners=True, reverse=False)

        # export data
        #data.uploadToCarto(table=table)
        data.uploadToDb(table=table)

        apiResponse.setHash(hash=data.userWeightsToHash())
        apiResponse.setFromCahce(cache=ApiResponse.no)
        apiResponse.setVarianceExplained(variance=data.getVarianceExplained())
        apiResponse.setFactorLoading(factors_loading=data.getFactorLoading())
        return apiResponse.getJson()


"""
@app.route('/detail',methods=["POST","GET"])
def handleDetail():
    apiResponse = ApiResponse(status=ApiResponse.ok)
    requestManager = RequestManager(request)
    table = getResultsTable(requestManager)
    kod = int(requestManager.getFromGet('kod'))
    page = requestManager.getFromGet('kategorie')


    data = DataHolder(select_keys=getSelectKeys(page), special_keys="kod,the_geom", api_key=app.config['API_KEY'])

    if not data.loadData(cacheFile=getCacheFilePath(page)):
        #data.downloadFromCarto()
        #data.removeOutliners(user_except_keys=exceptKeysForRemoveOutliners(page))
        #data.normalizeData()
        #data.reverseDataScale(user_except_keys=exceptKeysForReverseDataScale(page))
        #data.saveData(cacheFile=getCacheFilePath(page))
        apiResponse.setStatus(status=ApiResponse.error)
        apiResponse.setReason(reason="Data not in cache, please draw map first")
        return apiResponse.getJson()


    data.calculateQuantile(user_except_keys = [])
    out_data = data.getDataForKod(kod)

    apiResponse.setBins(bins=data.data_bins)
    apiResponse.setDetails(details=out_data)
    return apiResponse.getJson()


@app.route('/all',methods=["POST","GET"])
@app.route('/socialni',methods=["POST","GET"])
@app.route('/sluzby',methods=["POST","GET"])
@app.route('/nemovitosti',methods=["POST","GET"])
@app.route('/doprava',methods=["POST","GET"])
@app.route('/zivotni_prostredi',methods=["POST","GET"])
def handle():
    apiResponse = ApiResponse(status=ApiResponse.ok)

    try:
        requestManager = RequestManager(request)
        table = getResultsTable(requestManager)
        page = requestManager.getPage()
        userWeights = UserWeights(requestManager)
    except Exception as e:
        printConsole(e)
        apiResponse.setStatus(status=ApiResponse.error)
        apiResponse.setReason(reason="Not valid GET parameters or input data")
        return apiResponse.getJson()

    #try:self.data
    data = DataHolder(select_keys=getSelectKeys(page),special_keys="kod,the_geom", api_key=app.config['API_KEY'])
    data.setUserWeights(userWeights=userWeights.getUserWeights())

    # check wheather is already in db
    if checkIfHashExists(hash=data.userWeightsToHash(),table=table):
        apiResponse.setHash(hash=data.userWeightsToHash())
        apiResponse.setFromCahce(cache=ApiResponse.yes)
        return apiResponse.getJson()
    else:
        hash = data.userWeightsToHash()

        if not data.loadData(cacheFile=getCacheFilePath(requestManager.getPage())):
            data.downloadFromCarto()
            data.removeOutliners(user_except_keys=exceptKeysForRemoveOutliners(page))
            data.standardizeData()
            data.normalizeData()
            data.reverseDataScale(user_except_keys=exceptKeysForReverseDataScale(page))
            data.saveData(cacheFile=getCacheFilePath(requestManager.getPage()))

        # WeightsSum
        data.runWeightsSum()
        # WPCA
        data.setWPCAWeights()
        data.runWPCA()
        # normalize output

        # do not reverse if social
        # DO NOT KNOW WHY?!!!
        if(page == "socialni" or page=="all"):
            reverse = False
        else:
            reverse = True
        data.normalizeOutput(normalize=True, removeOutliners=True, reverse=False)

        # export data
        #data.uploadToCarto(table=table)
        data.uploadToDb(table=table)

        apiResponse.setHash(hash=data.userWeightsToHash())
        apiResponse.setFromCahce(cache=ApiResponse.no)
        apiResponse.setVarianceExplained(variance=data.getVarianceExplained())
        apiResponse.setFactorLoading(factors_loading=data.getFactorLoading())
        return apiResponse.getJson()

    #except Exception as e:
    #    apiResponse = ApiResponse(status=ApiResponse.error)
    #    apiResponse.setReason(reason=e)
    #    return apiResponse.getJson()


def getCacheFileFolder():
    return app.config['cacheFileFolder']

def getCacheFilePath(page):
    return app.config['cacheFileFolder']+str(page)+".bin"

def exceptKeysForRemoveOutliners(page):
    if page == "zivotni_prostredi":
        return ["blizkost_kontaminace","znecisteni_ovzdusi_prekroceni"]
    elif page == "nemovitosti":
        return ["zaplavove_uzemi_aktivni_score","v_obytne_zone","internet","v_residencni_ulici"]
    elif page == "socialni":
        return ["pocitova_mapa"]
    elif page == "sluzby":
        return []
    elif page == "doprava":
        return []
    elif page == "all":
        return exceptKeysForRemoveOutliners("zivotni_prostredi")+exceptKeysForRemoveOutliners("nemovitosti")+exceptKeysForRemoveOutliners("socialni")+exceptKeysForRemoveOutliners("sluzby")+exceptKeysForRemoveOutliners("doprava")
    else:
        Exception("not correct URL")

def exceptKeysForReverseDataScale(page):
    if page == "zivotni_prostredi":
        return ["plocha_zelene_osm"]
    elif page == "nemovitosti":
        return ["pomer_obytnych_budov","v_obytne_zone","internet","v_residencni_ulici"]
    elif page == "socialni":
        return ["facebook_navstevnost","pocitova_mapa","hustota_zalidneni"]
    elif page == "sluzby":
        return ["obchody_plocha"]
    elif page == "doprava":
        return []
    elif page == "all":
        return exceptKeysForReverseDataScale("zivotni_prostredi")+exceptKeysForReverseDataScale("nemovitosti")+exceptKeysForReverseDataScale("socialni")+exceptKeysForReverseDataScale("sluzby")+exceptKeysForReverseDataScale("doprava")
    else:
        Exception("not correct URL")

def getSelectKeys(page):
    if page == "zivotni_prostredi":
        return "blizkost_kontaminace,hluk_2012,plocha_zelene_osm,frekventovana_silnice_score,znecisteni_ovzdusi_prekroceni"
    elif page == "nemovitosti":
        return "zaplavove_uzemi_aktivni_score,cena_nemovitosti,pomer_obytnych_budov,okolni_pocet_podlazi,plocha_brownfiled,v_obytne_zone,v_residencni_ulici,internet"
    elif page == "socialni":
        return "hustota_zalidneni, podil_romske_narodnosti, podil_postproduktivni_obyvatel, facebook_navstevnost, kriminalita, pocitova_mapa"
    elif page == "sluzby":
        return "recyklokontejner_vzdalenost, posta_vzdalenost, potraviny_vzdalenost, hospodybary_vzdalenost, obchody_plocha, restaurace_vzdalenost"
    elif page == "doprava":
        return "mhd_vzdalenost, mhd_ceska, mhd_mendlak, mhd_skacelova"
    elif page == "all":
        return getSelectKeys("zivotni_prostredi")+","+getSelectKeys("nemovitosti")+","+getSelectKeys("socialni")+","+getSelectKeys("sluzby")+","+getSelectKeys("doprava")
    else:
        Exception("not correct URL")


def getResultsTable(requestManager):
    return "results_"+str(requestManager.getPage())
"""

def deleteFromCarto(table=None):
    if not table:
        Exception("Please specify table")
    cl = CartoDBAPIKey(app.config['API_KEY'], 'blue4world', host='cartodb.ml', protocol='http')
    sql = "DELETE FROM "+table
    cl.sql(sql)

def checkIfHashExists(hash=hash,table=None):
    try:
        cl = CartoDBAPIKey(app.config['API_KEY'], 'blue4world', host='cartodb.ml', protocol='http')
        cartoData = cl.sql("SELECT cartodb_id FROM "+table+" where hash = '"+hash+"' limit 1")
        i = 0
        for name in cartoData[u'rows']:
            i += 1
        if i == 0:
            return False
        else:
            return True

    except CartoDBException as e:
        print("some error ocurred", e)

if __name__ == "__flask__":
    app.run()






















