from cartodb import CartoDBAPIKey, CartoDBException
import numpy as np
from sklearn.decomposition import PCA as sklearnPCA
from sklearn.decomposition import KernelPCA as sklearnKernelPCA
import sklearn
import scipy.stats as stats
import pylab as plt
from wpca import WPCA, EMPCA
from PFA import PFA
import cPickle as pickle
import warnings
#import gc
import math
import psycopg2
import marshal
import api
import collections
import os.path
import pandas
import hashlib

class DataHolder:
    def __init__(self,select_keys="",special_keys="the_geom,kod,zpusob_vyuziti_bool",api_key='a50f67b019a990a1f29893ca6beab24028c2aea0'):
        self.API_KEY = api_key

        self.keys = self._stringToArray(select_keys)
        self.special_keys = self._stringToArray(special_keys)
        self.keys = self.keys + self._getSpecialKeys()

        self.data = dict()
        self.data_bins = dict()
        self.user_weights = None
        # pca
        self.pca = None
        self.pca_data = None
        self.pca_data_array = None
        # wpca
        self.wpca_data = None
        self.wpca = None
        self.wpca_weights = None

        self.wpca_data_array = None
        # sum
        self.sum_data = None

        warnings.filterwarnings("ignore", category=DeprecationWarning)
        warnings.filterwarnings("ignore", category=UserWarning)
        warnings.filterwarnings("ignore", category=RuntimeWarning)


    def saveData(self,cacheFile=None):
        if os.path.exists(cacheFile):
            api.printConsole("Not saving, already exists")
        else:
            api.printConsole("Saving data")
            #gc.disable()
            directory = api.app.config['cacheFileFolder']
            if not os.path.exists(directory):
                os.makedirs(directory)
            with open(cacheFile, 'wb') as f:
                pickle.dump(self.data, f,pickle.HIGHEST_PROTOCOL)
            #gc.enable()


    def loadData(self,cacheFile=None):
        if not os.path.exists(cacheFile):
            api.printConsole("Data not in cache")
            return False
        else:
            api.printConsole("Loading from cache")
            #gc.disable()
            with open(cacheFile, 'r') as f:
                self.data = pickle.load(f)
            #gc.enable()
            return True



    def downloadFromCarto(self):
        try:
            api.printConsole("Loading from network")
            cl = CartoDBAPIKey(self.API_KEY, 'blue4world', host='cartodb.ml', protocol='http')
            # only residental buildings
            # where zpusob_vyuziti_bool = 0
            cartoData = cl.sql('select '+",".join(self.keys)+' from budovy_index order by kod asc ')


            # init arrays
            for name in cartoData[u'fields']:
                name = name.encode('ascii','ignore')
                self.data[name] = []
            # fill with data
            for name in cartoData[u'rows']:
                for key in self.data.keys():
                    # if null put zero
                    if name[key]:
                        self.data[key].append(name[key])
                    else:
                        self.data[key].append(0)
            # print stats
            api.printConsole("Loaded keys: "+str(",".join(self.data.keys())))
            api.printConsole("Num of rows: "+str(self._getNumOfRows()))

        except CartoDBException as e:
            api.printConsole(("some error ocurred", e))


    def calculateQuantileForData(self,data):
        intervals = []
        for i in range(0, 7):
            intervals.append(np.percentile(np.array(data), 14.28*i))
        return intervals


    def calculateQuantile(self,user_except_keys = []):
        except_keys = self._getSpecialKeys() + user_except_keys
        for key in self.data.keys():
            if key not in except_keys:
                self.data_bins[key] = []
                for i in range(0, 7):
                    self.data_bins[key].append(np.percentile(np.array(self.data[key]), 14.28*i))


    def getDataForKod(self,kod):
        # find postition
        i = 0
        for key in self.data.keys():
            if key == "kod":
                for j in range(0,(len(self.data[key]))):
                    if(kod == self.data[key][j]):
                        break
                    i += 1

        data = {}
        except_keys = self._getSpecialKeys()
        for key in self.data.keys():
            if key not in except_keys:
                data[key] = self.data[key][i]
        return data

    def removeOutliners(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        for key in self.data.keys():
            if key not in except_keys:
                self.data[key] = self._removeOutliers(self.data[key])

    def normalizeData(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        for key in self.data.keys():
            if key not in except_keys:
                self.data[key] = sklearn.preprocessing.MinMaxScaler().fit_transform(self.data[key])
                self.data[key] = self._removeSmallerThanZero(self.data[key])

    # http://scikit-learn.org/stable/modules/generated/sklearn.preprocessing.StandardScaler.html#sklearn.preprocessing.StandardScaler
    def standardizeData(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        for key in self.data.keys():
            if key not in except_keys:
                self.data[key] = sklearn.preprocessing.StandardScaler().fit_transform(self.data[key])


    def reverseDataScale(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys()+user_except_keys
        for key in self.data.keys():
            if key not in except_keys:
                self.data[key] = self._substractOne(self.data[key])
                self.data[key] = self._removeSmallerThanZero(self.data[key])


    def calculateFactorLoadings(self):
        pca_factor_loadings = self.pca.components_.T * math.sqrt(self.pca.explained_variance_)
        wpca_factor_loadings = self.wpca.components_.T * math.sqrt(self.wpca.explained_variance_)
        api.printConsole("PCA factor loadings: "+str(pca_factor_loadings))
        api.printConsole("WCA factor loadings: " + str(wpca_factor_loadings))


    def _getDataMatrix(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        data = []
        for key in self.data.keys():
            if key not in except_keys:
                data.append(self.data[key])
        data = np.array(data)
        api.printConsole("Data shape is: " + str(data.shape))
        return data

    def setUserWeights(self,userWeights = { }):
        self.user_weights = userWeights

    def setWPCAWeights(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        weights_data = []
        userWeights = self.user_weights
        for key in self.data.keys():
            if key not in except_keys:
                if key in userWeights.keys():
                    weights_data.append([userWeights[key]] * self._getNumOfRows())
                else:
                    weights_data.append([0] * self._getNumOfRows())

        weights_data = np.array(weights_data)
        api.printConsole("Weights shape is: " + str(weights_data.shape))
        self.wpca_weights = weights_data

    def getNormalizedSumForKeys(self, keys = [],user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        sum = [0] * self._getNumOfRows()

        for key in keys:
            if key not in except_keys:
                    index = 0
                    for i in self.data[key]:
                        sum[index] += i
                        index += 1

        sum = sklearn.preprocessing.MinMaxScaler().fit_transform(sum)
        sum = self._removeSmallerThanZero(sum)
        return sum


    def runWeightsSum(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        userWeights= self.user_weights

        sum = [0]*self._getNumOfRows()

        for key in self.data.keys():
            if key not in except_keys:
                if key in userWeights.keys():
                    index = 0
                    for i in self.data[key]:
                        sum[index] += i * userWeights[key]
                        index += 1
        self.sum_data = sum

    # github.com/jakevdp/wpca
    def runWPCA(self,weights=None,user_except_keys=[]):
        api.printConsole("Running WPCA")
        except_keys = self._getSpecialKeys() + user_except_keys
        data = self._getDataMatrix(user_except_keys)

        kwds = {'weights': self.wpca_weights.T}
        self.wpca = WPCA(n_components=1).fit(data.T,**kwds)
        api.printConsole("Percentage of variance explained: " + str(self.wpca.explained_variance_ratio_))

        self.wpca_data = WPCA(n_components=1).fit_transform(data.T,**kwds)
        self.wpca_data_array = self._PCAresultToSimpleArray(self.wpca_data)

    def getVarianceExplained(self):
        return str(self.wpca.explained_variance_ratio_[0])

    def getFactorLoading(self):
        wpca_factor_loadings = self.wpca.components_.T * math.sqrt(self.wpca.explained_variance_)
        except_keys = self._getSpecialKeys()
        data = {}
        i = 0
        for key in self.data.keys():
            if key not in except_keys:
                data[key] = wpca_factor_loadings[i][0]
                i+=1
        return data


    def normalizeOutput(self,normalize=True,removeOutliners=True,reverse=True):

        if removeOutliners:
            if self.pca_data_array:
                self.pca_data_array = self._removeOutliers(self.pca_data_array)
            if self.wpca_data_array:
                self.wpca_data_array = self._removeOutliers(self.wpca_data_array)
            if self.sum_data:
                self.sum_data = self._removeOutliers(self.sum_data)

        if normalize:
            # normalize PCA
            if self.pca_data_array:
                self.pca_data_array = sklearn.preprocessing.MinMaxScaler().fit_transform(self.pca_data_array)
                if reverse:
                    self.pca_data_array = self._substractOne(self.pca_data_array)
                self.pca_data_array = self._removeSmallerThanZero(self.pca_data_array)

            # normalize WPCA
            if self.wpca_data_array:
                self.wpca_data_array = sklearn.preprocessing.MinMaxScaler().fit_transform(self.wpca_data_array)
                if reverse:
                    self.wpca_data_array = self._substractOne(self.wpca_data_array)
                self.wpca_data_array = self._removeSmallerThanZero(self.wpca_data_array)

            # normalize sum
            if self.sum_data:
                self.sum_data = sklearn.preprocessing.MinMaxScaler().fit_transform(self.sum_data)
                self.sum_data = self._removeSmallerThanZero(self.sum_data)


    # http://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html#sklearn.decomposition.PCA
    # http://scikit-learn.org/stable/modules/generated/sklearn.decomposition.FactorAnalysis.html
    def runPCA(self,user_except_keys=[],algorithm='PCA'):
        data = self._getDataMatrix(user_except_keys)

        if algorithm == "PCA":
            api.printConsole("Running PCA")
            self.pca = sklearnPCA(n_components=1, whiten=True)
        elif algorithm == "IncrementalPCA":
            api.printConsole("Running IncrementalPCA")
            self.pca = sklearn.decomposition.IncrementalPCA(n_components=1)
        elif algorithm == "TruncatedSVD":
            api.printConsole("Running TruncatedSVD")
            self.pca = sklearn.decomposition.TruncatedSVD(n_components=1)

        self.pca_data = self.pca.fit_transform(data.T)
        api.printConsole("Percentage of variance explained: "+str(self.pca.explained_variance_ratio_))
        #print "Components: "+str(self.pca.components_)
        self.pca_data_array = self._PCAresultToSimpleArray(self.pca_data)

    def showOutputPlot(self,type):
        if type == "PCA":
            data = self.pca_data_array
        else:
            data = self.wpca_data_array
        fig = plt.figure()
        ax = fig.add_subplot(1, 2, 1)
        ax.boxplot(data)
        ax.title.set_text(type)
        ax = fig.add_subplot(1, 2, 2)
        ax.hist(data)
        ax.title.set_text(type)
        plt.show()

    def exportToFile(self,filename="data.csv"):
        file = open(filename, "w")
        file.write(str(";".join(self.data.keys())))
        file.write(";pca;wpca\n")

        for i in range(0,self._getNumOfRows()):
            for key in self.data.keys():
                list = self.data[key]
                file.write(str(list[i]))
                file.write(";")
            file.write(str(self.pca_data_array[i]))
            file.write(";")
            file.write(str(self.wpca_data_array[i]))
            file.write("\n")

        file.close()


    def showKeysPlots(self,histPlots=False):
        except_keys = self._getSpecialKeys()
        i=1
        fig = plt.figure()
        for key in self.data.keys():
            if key not in except_keys:
                ax = fig.add_subplot(1,self._getNumOfKeys(),i)
                if histPlots:
                    ax.hist(self.data[key])
                else:
                    ax.boxplot(self.data[key])
                    ax.set_ylim([-0.2, 1.2])

                ax.title.set_text(key)
                i=i+1
        plt.show()

    def _getNumOfRows(self):
        return len(self.data[self.data.keys()[0]])

    def _getNumOfKeys(self):
        numOfKeys = 0
        for key in self.data.keys():
            if key not in self._getSpecialKeys():
                numOfKeys += 1
        return numOfKeys

    def _substractOne(self,array):
        return [1 - x for x in array]

    def _removeSmallerThanZero(self,list):
        newList = []
        for i in list:
            if i < 0.0:
                newList.append(0)
            elif i == 1.11022302463e-16:
                newList.append(0)
            else:
                newList.append(i)
        return newList

    def _getSpecialKeys(self):
        return self.special_keys

    # https://gist.github.com/vishalkuo/f4aec300cf6252ed28d3#file-outlier_removal-py-L3
    def _removeOutliers(self,x, outlierConstant=1.5):
        a = np.array(x)
        upper_quartile = np.percentile(a, 75)
        lower_quartile = np.percentile(a, 25)
        IQR = (upper_quartile - lower_quartile) * outlierConstant
        quartileSet = (lower_quartile - IQR, upper_quartile + IQR)
        resultList = []
        for i in a:
           if i >= quartileSet[1]:
              resultList.append(quartileSet[1])
           elif i <= quartileSet[0]:
              resultList.append(quartileSet[0])
           else:
             resultList.append(i)
        return resultList

    def _stringToArray(self,string):
        return string.split(",")

    def _PCAresultToSimpleArray(self,PCA_data):
        data = []
        for i in PCA_data:
            data.append(i[0])
        return data

    def userWeightsToHash(self):
        h = hashlib.new('ripemd160')
        h.update(str(self.user_weights))
        n = int(h.hexdigest(), base=16)
        n = n / 10000000000000000000000000000000000000000000
        return str(n)


    def _generateSqlForUpload(self,table,query_hash):
        sql = 'INSERT INTO ' + table + ' (kod, value, sum, the_geom, hash) VALUES'
        for i in range(0, self._getNumOfRows()):
            if self.user_weights["pouze_obytne"] == 1:
                if self.data["zpusob_vyuziti_bool"][i] == 0:
                    if i == 0:
                        sql += ' ('
                    else:
                        sql += ', ('
                    sql += str(self.data["kod"][i]) + ', '
                    sql += str(self.wpca_data_array[i]) + ', '
                    sql += str(self.sum_data[i]) + ', '
                    sql += "'" + str(self.data["the_geom"][i]) + "'" + ', '
                    sql += "'" + str(query_hash) + "'"
                    sql += ') '
            else:
                if i == 0:
                    sql += ' ('
                else:
                    sql += ', ('
                sql += str(self.data["kod"][i]) + ', '
                sql += str(self.wpca_data_array[i]) + ', '
                sql += str(self.sum_data[i]) + ', '
                sql += "'" + str(self.data["the_geom"][i]) + "'"  + ', '
                sql += "'" + str(query_hash) + "'"
                sql += ') '
        return sql

    def uploadToCarto(self,table=None):
        if not table:
            Exception("Please specify table")
        query_hash = self.userWeightsToHash()
        api.printConsole("Uploading to Carto")
        cl = CartoDBAPIKey(self.API_KEY, 'blue4world', host='cartodb.ml', protocol='http')
        sql = self._generateSqlForUpload(table,query_hash)
        response = cl.sql(sql)
        api.printConsole(response)


    def uploadToDb(self,table=None):
        api.printConsole("Uploading to database")
        try:
            conn = psycopg2.connect(
                "dbname='cartodb_dev_user_91beae7f-f361-4ae9-86cd-e9dc4ad27643_db' user='USER_HERE' host='localhost' password='PASSWORD_HERE'")
        except:
            api.printConsole("I am unable to connect to the database")

        sql = self._generateSqlForUpload(table,self.userWeightsToHash())
        cur = conn.cursor()
        cur.execute(sql)
        cur.execute("commit;")


    """
    def uploadDirectToDb(self,tablename="data"):
        try:
            conn = psycopg2.connect("dbname='cartodb_dev_user_91beae7f-f361-4ae9-86cd-e9dc4ad27643_db' user='USER_HERE' host='localhost' password='PASSWORD_HERE'")
        except:
            print "I am unable to connect to the database"

        cur = conn.cursor()

        #print "Create index"
        #cur.execute("CREATE UNIQUE INDEX index_kod on data (kod);")

        print "Update data"
        sql_data = []
        for i in range(0,self._getNumOfRows()):
            data = { }
            data["wpa"] = self.wpca_data_array[i]
            data["kod"] = self.data["kod"][i]
            sql_data.append(data)

        cur.executemany(
            '''
                UPDATE data
                SET
                    test = %(wpa)s
                WHERE
                    kod = %(kod)s
            ''',sql_data)
        cur.execute("commit;")
    """


    """ just testing
    def runPFA(self,user_except_keys=[]):
        except_keys = self._getSpecialKeys() + user_except_keys
        data = []
        for key in self.data.keys():
            if key not in except_keys:
                data.append(self.data[key])
        data = np.array(data)

        pfa = PFA(n_features=1)
        pfa.fit(data)

        # To get the transformed matrix
        X = pfa.features_
        print X

        # To get the column indices of the kept features
        column_indices = pfa.indices_
        print column_indices
    """
