from cartodb import CartoDBAPIKey, CartoDBException
import numpy as np
from sklearn.decomposition import PCA as sklearnPCA
from sklearn.decomposition import KernelPCA as sklearnKernelPCA
import sklearn
import scipy.stats as stats
import pylab as pl
from wpca import WPCA, EMPCA
from data_holder import DataHolder

# user weights
userWeights = {}
userWeights["blizkost_kontaminace"] = 1
userWeights["hluk_2012"] = 1
userWeights["plocha_zelene_osm"] = 1
userWeights["frekventovana_silnice_score"] = 1
userWeights["znecisteni_ovzdusi_prekroceni"] = 1

### zivotni prostredi
data = DataHolder(select_keys="blizkost_kontaminace,hluk_2012,plocha_zelene_osm,frekventovana_silnice_score,znecisteni_ovzdusi_prekroceni")
data.downloadFromCarto(cacheFile="zivotni_prostredi.pickle",fromCache=True)
# data preparation
data.removeOutliners(user_except_keys=["blizkost_kontaminace"])
#data.standardizeData()
data.normalizeData()
data.reverseDataScale(user_except_keys=["plocha_zelene_osm"])
#show data
#data.showKeysPlots(histPlots=True)

# PCA
data.runPCA()
# WPCA
data.setWPCAWeights(userWeights=userWeights)
data.runWPCA()
# normalize output
data.normalizeOutput(normalize=True,removeOutliners=False)

#show output data
#data.calculateFactorLoadings()
#data.showOutputPlot("PCA")
#data.showOutputPlot("WPCA")


# export data
#data.uploadToCarto(table="results_zivotni_prostredi")
#data.uploadDirectToDb(tablename="data")
#data.exportToFile(filename="zivotni_prostredi.csv")



"""
### nemovistosti
data = DataHolder(select_keys="zaplavove_uzemi_aktivni_score,cena_nemovitosti,pomer_obytnych_budov,okolni_pocet_podlazi,plocha_brownfiled,v_obytne_zone")
data.downloadFromCarto(cacheFile="nemovitosti.pickle",fromCache=False)
data.removeOutliners(user_except_keys=["zaplavove_uzemi_aktivni_score","v_obytne_zone"])
data.normalizeData()
data.reverseDataScale(user_except_keys=["pomer_obytnych_budov","v_obytne_zone"])
data.showKeysBoxPlots()
data.runDimensionReduction(normalize=True,removeOutliners=False,algorithm='PCA')
data.showPCAPlot()
data.exportToFile(filename="nemovitosti.csv")



### socialni
data = DataHolder(select_keys="hustota_zalidneni, podil_romske_narodnosti, podil_postproduktivni_obyvatel, facebook_navstevnost")
data.downloadFromCarto(cacheFile="socialni.pickle",fromCache=False)
data.removeOutliners(user_except_keys=[])
data.normalizeData()
data.reverseDataScale(user_except_keys=["hustota_zalidneni","podil_romske_narodnosti","podil_postproduktivni_obyvatel"])
data.showKeysBoxPlots()
data.runDimensionReduction(normalize=True,removeOutliners=False,algorithm='PCA')
data.showPCAPlot()
data.exportToFile(filename="socialni.csv")
"""

"""
### all
data = DataHolder(select_keys="hustota_zalidneni, podil_romske_narodnosti, podil_postproduktivni_obyvatel, facebook_navstevnost, zaplavove_uzemi_aktivni_score,cena_nemovitosti,pomer_obytnych_budov,okolni_pocet_podlazi,plocha_brownfiled,v_obytne_zone, blizkost_kontaminace,hluk_2012,plocha_zelene_osm,frekventovana_silnice_score,znecisteni_ovzdusi_prekroceni")
data.downloadFromCarto(cacheFile="all.pickle",fromCache=False)
data.removeOutliners(user_except_keys=["blizkost_kontaminace","zaplavove_uzemi_aktivni_score","v_obytne_zone"])
data.normalizeData()
data.reverseDataScale(user_except_keys=["hustota_zalidneni","podil_romske_narodnosti","podil_postproduktivni_obyvatel","plocha_zelene_osm","pomer_obytnych_budov","v_obytne_zone"])
data.showKeysBoxPlots()
data.runDimensionReduction(normalize=True,removeOutliners=False,algorithm='PCA')
data.showPCAPlot()
data.exportToFile(filename="all.csv")
"""